@@ -102,6 +102,8 @@ void KSudoku::onCompleted(bool isCorrect, const QTime& required, bool withHelp)
 		else
 			msg = i18nc("The two parameters are strings like '2 minutes' or '1 second'.", "Congratulations! You made it in %1 and %2.", i18np("1 minute", "%1 minutes", mins), i18np("1 second", "%1 seconds", secs));
 
+    onModified(true); // make sure buttons have the correct enabled state
+
 	KMessageBox::information(this, msg);
 
 }
@@ -114,9 +116,9 @@ void KSudoku::onCompleted(bool isCorrect, const QTime& required, bool withHelp)
 // // 		m = view->status();
 // // 	if(currentView())
 // // 		m = currentView()->status();
-// 
+//
 // 	// TODO fix this: add new status bar generation code
-// 
+//
 // 	statusBar()->showMessage(m);
 // }
 
@@ -236,8 +238,8 @@ void KSudoku::updateShapesList()
 void KSudoku::startGame(const Game& game) {
 	m_welcomeScreen->hide();
 	endCurrentGame();
-	
-	
+
+
 	KsView* view = new KsView(game, m_gameActions, this);
 
 	view->setValueListWidget(m_valueListWidget);
@@ -352,10 +354,10 @@ void KSudoku::startGame(const Game& game) {
 
 void KSudoku::endCurrentGame() {
 	m_valueListWidget->hide();
-	
+
 	delete m_gameUI;
 	m_gameUI = 0;
-	
+
 	adaptActions2View();
 
 }
@@ -457,6 +459,7 @@ void KSudoku::setupActions()
 	setAcceptDrops(true);
 
 	KStandardGameAction::gameNew(this, SLOT(gameNew()), actionCollection());
+	KStandardGameAction::restart(this, SLOT(gameRestart()), actionCollection());
 	KStandardGameAction::load(this, SLOT(gameOpen()), actionCollection());
 	m_gameSave = KStandardGameAction::save(this, SLOT(gameSave()), actionCollection());
 	m_gameSaveAs = KStandardGameAction::saveAs(this, SLOT(gameSaveAs()), actionCollection());
@@ -545,7 +548,15 @@ void KSudoku::adaptActions2View() {
 
 	m_gameSave->setEnabled(game.isValid());
 	m_gameSaveAs->setEnabled(game.isValid());
+	action("game_new")->setEnabled(game.isValid());
+	action("game_restart")->setEnabled(game.isValid());
+	action("game_print")->setEnabled(game.isValid());
 	if(game.isValid()) {
+		bool isEnterPuzzleMode = !game.puzzle()->hasSolution();
+		action("move_hint")->setVisible(!isEnterPuzzleMode);
+		action("move_solve")->setVisible(!isEnterPuzzleMode);
+		action("move_dub_puzzle")->setVisible(isEnterPuzzleMode);
+
 		action("move_undo")->setEnabled(game.canUndo());
 		action("move_redo")->setEnabled(game.canRedo());
 
@@ -556,9 +567,9 @@ void KSudoku::adaptActions2View() {
 		action("move_undo")->setEnabled(false);
 		action("move_redo")->setEnabled(false);
 
-		action("move_hint")->setEnabled(false);
-		action("move_solve")->setEnabled(false);
-		action("move_dub_puzzle")->setEnabled(false);
+		action("move_hint")->setVisible(false);
+		action("move_solve")->setVisible(false);
+		action("move_dub_puzzle")->setVisible(false);
 	}
 }
 
@@ -567,6 +578,8 @@ void KSudoku::onModified(bool /*isModified*/) {
 	if(game.isValid()) {
 		action("move_undo")->setEnabled(game.canUndo());
 		action("move_redo")->setEnabled(game.canRedo());
+		action("move_hint")->setEnabled(!game.allValuesSetAndUsable());
+		action("move_solve")->setEnabled(!game.wasFinished());
 	}
 }
 
@@ -647,8 +660,8 @@ void KSudoku::gameNew()
 	if(!m_gameUI->game().wasFinished()) {
 		if(KMessageBox::questionYesNo(this,
 	                              i18n("Do you really want to end this game in order to start a new one?"),
-	                              i18nc("window title", "Restart Game"),
-	                              KGuiItem(i18nc("button label", "Restart Game")),
+	                              i18nc("window title", "New Game"),
+	                              KGuiItem(i18nc("button label", "New Game")),
 	                              KStandardGuiItem::cancel() ) != KMessageBox::Yes)
 			return;
 	}
@@ -656,6 +669,26 @@ void KSudoku::gameNew()
 	showWelcomeScreen();
 }
 
+void KSudoku::gameRestart()
+{
+	if (!currentView()) return;
+
+	auto game = currentGame();
+
+	// only show question when the current game hasn't been finished until now
+	if (!game.wasFinished()) {
+		if (KMessageBox::questionYesNo(this,
+                                i18n("Do you really want to restart this game?"),
+                                i18nc("window title", "Restart Game"),
+                                KGuiItem(i18nc("button label", "Restart Game")),
+                                KStandardGuiItem::cancel() ) != KMessageBox::Yes) {
+			return;
+		}
+	}
+
+	game.restart();
+}
+
 void KSudoku::gameOpen()
 {
 	// this slot is called whenever the Game->Open menu is selected,
@@ -811,7 +844,10 @@ void KSudoku::difficultyChanged (int difficulty)
 		i18n("Warning: The Unlimited difficulty level has no limit on "
 		     "how many guesses or branch points are required to solve "
 		     "the puzzle and there is no lower limit on how soon "
-		     "guessing becomes necessary."),
+		     "guessing becomes necessary.\n\n"
+		     "Please also note that the generation of this type of puzzle "
+		     "might take much longer than other ones. During this time "
+		     "KSudoku will not respond."),
 		i18n("Warning"), "WarningReUnlimited");
     }
 }
@@ -851,7 +887,9 @@ void KSudoku::enableMessages()
 {
 	// Enable all messages that the user has marked "Do not show again".
 	int result = KMessageBox::questionYesNo(this,
-					i18n("Enable all messages"));
+					i18n("This will enable all the dialogs that you had disabled by marking "
+						 "the 'Do not show this message again' option.\n\n"
+						 "Do you want to continue?"));
 	if (result == KMessageBox::Yes) {
 		KMessageBox::enableAllMessages();
 		KSharedConfig::openConfig()->sync();	// Save the changes to disk.
